@@ -3,7 +3,6 @@
 import click
 
 from .list_models import list_models
-from .quick_ref import quick_reference
 from .run import run
 from .update_registry import update_registry
 
@@ -15,7 +14,6 @@ def create_command_group() -> click.Group:
 
     # Add all commands to the group
     group.add_command(run)
-    group.add_command(quick_reference)
     group.add_command(update_registry)
     group.add_command(list_models)
 
@@ -25,7 +23,6 @@ def create_command_group() -> click.Group:
 # Export commands for easy importing
 __all__ = [
     "run",
-    "quick_reference",
     "update_registry",
     "list_models",
     "create_command_group",
