@@ -252,7 +252,20 @@ class PathSecurityError(SecurityErrorBase):
 class TaskTemplateError(CLIError):
     """Base class for task template-related errors."""
 
-    pass
+    def __init__(
+        self,
+        message: str,
+        context: Optional[Dict[str, Any]] = None,
+        exit_code: int = ExitCode.VALIDATION_ERROR,
+    ) -> None:
+        """Initialize task template error.
+
+        Args:
+            message: Error message
+            context: Additional error context
+            exit_code: Exit code (defaults to VALIDATION_ERROR)
+        """
+        super().__init__(message, context=context, exit_code=exit_code)
 
 
 class TaskTemplateSyntaxError(TaskTemplateError):
@@ -285,13 +298,161 @@ class TaskTemplateVariableError(TaskTemplateError):
 class TemplateValidationError(TaskTemplateError):
     """Raised when template validation fails."""
 
-    pass
+    def __init__(
+        self,
+        message: str,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize template validation error.
+
+        Args:
+            message: Error message
+            context: Additional error context
+        """
+        super().__init__(
+            message,
+            context=context,
+            exit_code=ExitCode.VALIDATION_ERROR,
+        )
 
 
 class SystemPromptError(TaskTemplateError):
     """Raised when there are issues with system prompt loading or processing."""
 
-    pass
+    def __init__(
+        self,
+        message: str,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize system prompt error.
+
+        Args:
+            message: Error message
+            context: Additional error context
+        """
+        super().__init__(
+            message,
+            context=context,
+            exit_code=ExitCode.VALIDATION_ERROR,
+        )
+
+
+# PDF and User Data support error classes
+
+
+class TemplateBinaryError(TaskTemplateError):
+    """Raised when template tries to access .content on a binary/user-data file."""
+
+    def __init__(
+        self,
+        message: str,
+        alias: Optional[str] = None,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize template binary error.
+
+        Args:
+            message: Error message
+            alias: File alias that caused the error
+            context: Additional error context
+        """
+        context = context or {}
+        if alias:
+            context["alias"] = alias
+            context.setdefault(
+                "details",
+                f"Template variable '{alias}.content' is not available for binary files",
+            )
+            context.setdefault(
+                "troubleshooting",
+                [
+                    f"Use '{alias}.name' to access the filename",
+                    f"Use '{alias}.size' to access the file size",
+                    "Mention the file verbally in your template instead",
+                    "Consider using 'prompt:' target for text files",
+                ],
+            )
+
+        super().__init__(message, context=context)
+
+
+class InsecureURLRejected(SecurityErrorBase):
+    """Raised when a URL violates security constraints."""
+
+    def __init__(
+        self,
+        message: str,
+        url: Optional[str] = None,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize insecure URL error.
+
+        Args:
+            message: Error message
+            url: URL that was rejected
+            context: Additional error context
+        """
+        context = context or {}
+        if url:
+            context["url"] = url
+            context.setdefault(
+                "details",
+                "The URL violates security constraints and was rejected",
+            )
+            context.setdefault(
+                "troubleshooting",
+                [
+                    "Use HTTPS URLs only",
+                    "Avoid private IP ranges (RFC-1918, loopback, link-local)",
+                    "Use --allow-insecure-url to explicitly allow specific URLs",
+                    "Set OSTRUCT_STRICT_URLS=0 to disable URL validation",
+                ],
+            )
+
+        super().__init__(
+            message,
+            context=context,
+        )
+
+
+class UserDataNotSupportedError(CLIError):
+    """Raised when user-data files are used with non-vision models."""
+
+    def __init__(
+        self,
+        message: str,
+        model_id: Optional[str] = None,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize user data not supported error.
+
+        Args:
+            message: Error message
+            model_id: Model that doesn't support user data
+            context: Additional error context
+        """
+        context = context or {}
+        if model_id:
+            context["model_id"] = model_id
+            context.setdefault(
+                "details",
+                f"Model '{model_id}' does not support user-data files",
+            )
+            context.setdefault(
+                "troubleshooting",
+                [
+                    "Use a vision-enabled model (e.g., gpt-4o, gpt-4.1)",
+                    "Convert files to text and use 'prompt:' target instead",
+                    "Use 'fs:' target for File Search RAG workflow",
+                    "Check model capabilities with 'ostruct list-models'",
+                ],
+            )
+
+        super().__init__(
+            message,
+            context=context,
+            exit_code=ExitCode.VALIDATION_ERROR,
+        )
 
 
 class SchemaError(CLIError):
@@ -561,7 +722,7 @@ class APIErrorMapper:
         ):
             return PromptTooLargeError(
                 f"Prompt exceeds model context window (128,000 token limit). "
-                f"Tip: Use explicit file routing (-fc for code, -fs for docs, -ft for config). "
+                f"Tip: Use explicit file routing (--file ci:data for code, --file fs:docs for docs, --file config for config). "
                 f"Original error: {error}"
             )
 
@@ -630,7 +791,7 @@ class APIErrorMapper:
             if "upload" in error_msg or "vector_store" in error_msg:
                 return FileSearchUploadError(
                     f"File Search upload failed: {error}. "
-                    f"This can be intermittent - retry with --file-search-retry-count option."
+                    f"This can be intermittent - retry with --fs-retries option."
                 )
             return FileSearchError(f"File Search error: {error}")
 
@@ -733,7 +894,17 @@ def handle_error(e: Exception) -> None:
         msg = f"Model creation error: {str(e)}"
         exit_code = ExitCode.SCHEMA_ERROR
     elif isinstance(e, click.UsageError):
-        msg = f"Usage error: {str(e)}"
+        error_msg = str(e)
+
+        # Enhance usage error messages with helpful guidance
+        if "Missing parameter" in error_msg:
+            if "task_template" in error_msg:
+                msg = f"Usage error: {error_msg}\n\nTry 'ostruct run --help' for usage information, 'ostruct --quick-ref' for examples, or 'ostruct run --help-debug' for troubleshooting help."
+            else:
+                msg = f"Usage error: {error_msg}\n\nTry 'ostruct run --help' for usage information or 'ostruct --quick-ref' for examples."
+        else:
+            msg = f"Usage error: {error_msg}"
+
         exit_code = ExitCode.USAGE_ERROR
     elif isinstance(e, SchemaFileError):
         msg = str(e)  # Use existing __str__ formatting
@@ -800,3 +971,135 @@ __all__ = [
     "InvalidResponseFormatError",
     "handle_error",
 ]
+
+# Download-specific error classes for Task 3
+
+
+class DownloadError(CLIError):
+    """Base class for download-related errors."""
+
+    def __init__(
+        self,
+        message: str,
+        context: Optional[Dict[str, Any]] = None,
+        exit_code: int = ExitCode.API_ERROR,
+    ) -> None:
+        """Initialize download error.
+
+        Args:
+            message: Error message
+            context: Additional error context
+            exit_code: Exit code for the error
+        """
+        super().__init__(message, context=context, exit_code=exit_code)
+
+
+class DownloadPermissionError(DownloadError):
+    """Raised when download fails due to permission issues."""
+
+    def __init__(
+        self,
+        directory: str,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize permission error.
+
+        Args:
+            directory: Directory that caused the permission error
+            context: Additional error context
+        """
+        context = context or {}
+        context.update(
+            {
+                "directory": directory,
+                "details": "Unable to write to the specified download directory",
+                "troubleshooting": [
+                    f"Check write permissions for directory: {directory}",
+                    "Verify the directory exists and is accessible",
+                    "Try using a different download directory with --ci-download-dir",
+                    "Check if the parent directory exists and is writable",
+                    "Ensure sufficient disk space is available",
+                ],
+            }
+        )
+
+        message = f"Permission denied when writing to download directory: {directory}"
+        super().__init__(
+            message, context=context, exit_code=ExitCode.FILE_ERROR
+        )
+
+
+class DownloadNetworkError(DownloadError):
+    """Raised when download fails due to network issues."""
+
+    def __init__(
+        self,
+        file_id: str,
+        original_error: Optional[Exception] = None,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize network error.
+
+        Args:
+            file_id: File ID that failed to download
+            original_error: Original exception that caused the failure
+            context: Additional error context
+        """
+        context = context or {}
+        context.update(
+            {
+                "file_id": file_id,
+                "details": "Network error occurred while downloading file from OpenAI",
+                "troubleshooting": [
+                    "Check your internet connection",
+                    "Verify OpenAI API is accessible",
+                    "Try the download again in a few moments",
+                    "Check if your API key has the necessary permissions",
+                    "Ensure the file ID is valid and not expired",
+                ],
+            }
+        )
+
+        if original_error:
+            context["original_error"] = str(original_error)
+
+        message = f"Network error downloading file {file_id}"
+        if original_error:
+            message += f": {original_error}"
+
+        super().__init__(message, context=context)
+
+
+class DownloadFileNotFoundError(DownloadError):
+    """Raised when a file to download is not found."""
+
+    def __init__(
+        self,
+        file_id: str,
+        context: Optional[Dict[str, Any]] = None,
+    ) -> None:
+        """Initialize file not found error.
+
+        Args:
+            file_id: File ID that was not found
+            context: Additional error context
+        """
+        context = context or {}
+        context.update(
+            {
+                "file_id": file_id,
+                "details": "The requested file was not found or is no longer available",
+                "troubleshooting": [
+                    "Verify the file ID is correct",
+                    "Check if the file was generated in this session",
+                    "Ensure the Code Interpreter execution completed successfully",
+                    "Try running the analysis again to regenerate the file",
+                    "Check if the file has expired (files have limited lifetime)",
+                ],
+            }
+        )
+
+        message = f"File not found for download: {file_id}"
+        super().__init__(
+            message, context=context, exit_code=ExitCode.FILE_ERROR
+        )
