@@ -5,7 +5,7 @@ import logging
 import os
 from enum import Enum
 from pathlib import Path
-from typing import Any, Optional
+from typing import Any, Iterator, Optional
 
 from .errors import FileReadError, OstructFileNotFoundError, PathSecurityError
 from .security import SecurityManager
@@ -18,9 +18,12 @@ class FileRoutingIntent(Enum):
     to provide appropriate warnings and optimizations.
     """
 
-    TEMPLATE_ONLY = "template_only"  # -ft, --fta, legacy -f, -d
-    CODE_INTERPRETER = "code_interpreter"  # -fc, --fca
-    FILE_SEARCH = "file_search"  # -fs, --fsa
+    TEMPLATE_ONLY = "template_only"  # --file [alias] (template access only)
+    CODE_INTERPRETER = "code_interpreter"  # --file ci:[alias]
+    FILE_SEARCH = "file_search"  # --file fs:[alias]
+    USER_DATA = (
+        "user_data"  # --file ud:[alias] (user-data files for vision models)
+    )
 
 
 logger = logging.getLogger(__name__)
@@ -30,7 +33,8 @@ class FileInfo:
     """Represents a file with metadata and content.
 
     This class provides access to file metadata (path, size, etc.) and content,
-    with caching support for efficient access.
+    with caching support for efficient access. Implements the file-sequence protocol
+    by being iterable (yields itself) while maintaining scalar access to properties.
 
     Args:
         path: Path to the file
@@ -77,16 +81,40 @@ class FileInfo:
         self.routing_type = routing_type
         self.routing_intent = routing_intent
 
+        # TSES v2.0 fields for alias tracking
+        self.parent_alias: Optional[str] = (
+            None  # CLI alias this file came from
+        )
+        self.relative_path: Optional[str] = (
+            None  # Path relative to attachment root
+        )
+        self.base_path: Optional[str] = None  # Base path of attachment
+        self.from_collection: bool = False  # Whether file came from --collect
+        self.attachment_type: str = (
+            "file"  # Original attachment type: "file", "dir", or "collection"
+        )
+
+        # For URL files, add is_url property
+        self._is_url: bool = False
+
         logger.debug(
             "Creating FileInfo for path: %s, routing_type: %s",
             path,
             self.routing_type,
         )
 
-        # Validate path
+        # Validate path – skip local checks for remote URLs
         if not path:
             raise ValueError("Path cannot be empty")
 
+        if str(path).startswith(("http://", "https://")):
+            # Remote URL – mark as such and bypass local filesystem validation
+            self._is_url = True
+            logger.debug(
+                "Remote URL detected, skipping filesystem validation: %s", path
+            )
+            return
+
         try:
             # This will raise PathSecurityError if path is not allowed
             # And FileNotFoundError if the file doesn't exist
@@ -149,6 +177,39 @@ class FileInfo:
                 f"Permission denied: {os.path.basename(str(path))}"
             ) from e
 
+    def __iter__(self) -> Iterator["FileInfo"]:
+        """Make FileInfo iterable by yielding itself.
+
+        This implements the file-sequence protocol, allowing single files
+        to be treated uniformly with file collections in templates.
+
+        Returns:
+            Iterator that yields this FileInfo instance
+        """
+        yield self
+
+    @property
+    def first(self) -> "FileInfo":
+        """Get the first file in the sequence (itself for single files).
+
+        This provides a uniform interface with FileInfoList.first,
+        allowing templates to use .first regardless of whether they're
+        dealing with a single file or a collection.
+
+        Returns:
+            This FileInfo instance
+        """
+        return self
+
+    @property
+    def is_collection(self) -> bool:
+        """Indicate whether this is a collection of files.
+
+        Returns:
+            False, since FileInfo represents a single file
+        """
+        return False
+
     @property
     def path(self) -> str:
         """Get the path relative to security manager's base directory.
@@ -156,25 +217,52 @@ class FileInfo:
         Returns a path relative to the security manager's base directory.
         This ensures consistent path handling across the entire codebase.
 
+        For paths outside the base directory but within allowed directories,
+        returns the absolute path.
+
         Example:
             security_manager = SecurityManager(base_dir="/base")
             file_info = FileInfo("/base/file.txt", security_manager)
             print(file_info.path)  # Outputs: "file.txt"
 
+            # With allowed directory outside base:
+            file_info = FileInfo("/tmp/file.txt", security_manager)
+            print(file_info.path)  # Outputs: "/tmp/file.txt"
+
         Returns:
-            str: Path relative to security manager's base directory
+            str: Path relative to security manager's base directory, or absolute path
+                 if outside base directory but within allowed directories
 
         Raises:
-            ValueError: If the path is not within the base directory
+            ValueError: If the path is not within the base directory or allowed directories
         """
+        abs_path = Path(self.abs_path)
+        base_dir = Path(self.__security_manager.base_dir)
+
         try:
-            abs_path = Path(self.abs_path)
-            base_dir = Path(self.__security_manager.base_dir)
             return str(abs_path.relative_to(base_dir))
-        except ValueError as e:
-            logger.error("Error making path relative: %s", e)
+        except ValueError:
+            # Path is outside base_dir, check if it's allowed by enhanced security model
+            try:
+                if self.__security_manager.is_path_allowed_enhanced(abs_path):
+                    logger.debug(
+                        "Path outside base_dir but allowed, returning absolute path: %s",
+                        abs_path,
+                    )
+                    return str(abs_path)
+            except Exception:
+                # In strict mode, is_path_allowed_enhanced() raises exceptions
+                # If we reach this except block, the path is not allowed
+                pass
+
+            # Should never reach here if SecurityManager validation was done properly
+            logger.error(
+                "Error making path relative: %s is not within base directory %s",
+                abs_path,
+                base_dir,
+            )
             raise ValueError(
-                f"Path {abs_path} must be within base directory {base_dir}"
+                f"Path {abs_path} is not within base directory {base_dir}"
             )
 
     @path.setter
@@ -216,6 +304,9 @@ class FileInfo:
     @property
     def size(self) -> Optional[int]:
         """Get file size in bytes."""
+        if self.is_url:
+            return None
+
         if self.__size is None:
             try:
                 size = os.path.getsize(self.abs_path)
@@ -233,6 +324,9 @@ class FileInfo:
     @property
     def mtime(self) -> Optional[float]:
         """Get file modification time as Unix timestamp."""
+        if self.is_url:
+            return None
+
         if self.__mtime is None:
             try:
                 mtime = os.path.getmtime(self.abs_path)
@@ -257,7 +351,22 @@ class FileInfo:
         Raises:
             FileReadError: If the file cannot be read, wrapping the underlying cause
                          (FileNotFoundError, UnicodeDecodeError, etc)
+            TemplateBinaryError: If trying to access content of a user-data file
         """
+        # Check for user-data files and block content access
+        if self.routing_intent == FileRoutingIntent.USER_DATA:
+            from .errors import TemplateBinaryError
+
+            # Try to get the alias from parent_alias if available
+            alias = getattr(self, "parent_alias", None) or "file"
+
+            raise TemplateBinaryError(
+                f"Cannot access .content on user-data file '{self.path}'. "
+                f"User-data files are sent directly to vision models and not "
+                f"included in the template text.",
+                alias=alias,
+            )
+
         # Add warning for large template-only files accessed via .content
         # Use intent-based logic with fallback to routing_type for backward compatibility
         template_only_intents = {FileRoutingIntent.TEMPLATE_ONLY}
@@ -280,7 +389,7 @@ class FileInfo:
                 f"File '{self.path}' ({self.size / 1024:.1f}KB) was routed for template-only access "
                 f"but its .content is being accessed. This will include the entire file content "
                 f"in the prompt sent to the AI. For large files intended for analysis or search, "
-                f"consider using -fc (Code Interpreter) or -fs (File Search) to optimize token usage, "
+                f"consider using --file ci:data (Code Interpreter) or --file fs:docs (File Search) to optimize token usage, "
                 f"cost, and avoid exceeding model context limits."
             )
 
@@ -338,6 +447,11 @@ class FileInfo:
         """Prevent setting hash directly."""
         raise AttributeError("Cannot modify hash directly")
 
+    @property
+    def is_url(self) -> bool:
+        """Check if this is a URL file."""
+        return self._is_url
+
     @property
     def exists(self) -> bool:
         """Check if the file exists.
@@ -411,6 +525,11 @@ class FileInfo:
         security_manager: SecurityManager,
         routing_type: Optional[str] = None,
         routing_intent: Optional[FileRoutingIntent] = None,
+        parent_alias: Optional[str] = None,
+        relative_path: Optional[str] = None,
+        base_path: Optional[str] = None,
+        from_collection: bool = False,
+        attachment_type: str = "file",
     ) -> "FileInfo":
         """Create FileInfo instance from path.
 
@@ -419,6 +538,11 @@ class FileInfo:
             security_manager: Security manager for path validation
             routing_type: How the file was routed (e.g., 'template', 'code-interpreter')
             routing_intent: The intended use of the file in the pipeline
+            parent_alias: CLI alias this file came from (for TSES)
+            relative_path: Path relative to attachment root (for TSES)
+            base_path: Base path of attachment (for TSES)
+            from_collection: Whether file came from --collect (for TSES)
+            attachment_type: Original attachment type: "file", "dir", or "collection" (for TSES)
 
         Returns:
             FileInfo instance
@@ -427,13 +551,22 @@ class FileInfo:
             FileNotFoundError: If file does not exist
             PathSecurityError: If path is not allowed
         """
-        return cls(
+        file_info = cls(
             path,
             security_manager,
             routing_type=routing_type,
             routing_intent=routing_intent,
         )
 
+        # Set TSES fields
+        file_info.parent_alias = parent_alias
+        file_info.relative_path = relative_path
+        file_info.base_path = base_path
+        file_info.from_collection = from_collection
+        file_info.attachment_type = attachment_type
+
+        return file_info
+
     def __str__(self) -> str:
         """String representation showing path."""
         return f"FileInfo({self.__path})"
@@ -459,6 +592,22 @@ class FileInfo:
             object.__setattr__(self, name, value)
             return
 
+        # Allow setting TSES fields
+        if name in (
+            "parent_alias",
+            "relative_path",
+            "base_path",
+            "from_collection",
+            "attachment_type",
+        ):
+            object.__setattr__(self, name, value)
+            return
+
+        # Allow setting _is_url at any time (idempotent)
+        if name == "_is_url":
+            object.__setattr__(self, name, value)
+            return
+
         # Allow setting private attributes from internal methods
         if name.startswith("_FileInfo__") and self._is_internal_call():
             object.__setattr__(self, name, value)
