@@ -10,25 +10,34 @@ import logging
 import os
 import time
 from pathlib import Path
-from typing import Any, Dict, List
+from typing import TYPE_CHECKING, Any, Dict, List, Optional
 
 from openai import AsyncOpenAI
 
+if TYPE_CHECKING:
+    from .upload_manager import SharedUploadManager
+
 logger = logging.getLogger(__name__)
 
 
 class FileSearchManager:
     """Manager for File Search vector store operations with retry logic."""
 
-    def __init__(self, client: AsyncOpenAI):
+    def __init__(
+        self,
+        client: AsyncOpenAI,
+        upload_manager: Optional["SharedUploadManager"] = None,
+    ) -> None:
         """Initialize File Search manager.
 
         Args:
             client: AsyncOpenAI client instance
+            upload_manager: Optional shared upload manager for deduplication
         """
         self.client = client
         self.uploaded_file_ids: List[str] = []
         self.created_vector_stores: List[str] = []
+        self.upload_manager = upload_manager
 
     async def create_vector_store_with_retry(
         self,
@@ -304,7 +313,7 @@ class FileSearchManager:
         )
         return False
 
-    def build_tool_config(self, vector_store_id: str) -> dict:
+    def build_tool_config(self, vector_store_id: str) -> Dict[str, Any]:
         """Build File Search tool configuration.
 
         Creates a tool configuration compatible with the OpenAI Responses API
@@ -321,6 +330,63 @@ class FileSearchManager:
             "vector_store_ids": [vector_store_id],
         }
 
+    async def create_vector_store_from_shared_manager(
+        self,
+        vector_store_name: str = "ostruct_vector_store",
+        max_retries: int = 3,
+        retry_delay: float = 1.0,
+    ) -> str:
+        """Create vector store and populate with files from shared upload manager.
+
+        Args:
+            vector_store_name: Name for the vector store
+            max_retries: Maximum retry attempts
+            retry_delay: Delay between retries
+
+        Returns:
+            Vector store ID
+
+        Raises:
+            Exception: If vector store creation or file upload fails
+        """
+        if not self.upload_manager:
+            logger.warning("No shared upload manager available")
+            # Fall back to creating empty vector store
+            return await self.create_vector_store_with_retry(
+                vector_store_name, max_retries, retry_delay
+            )
+
+        # Get file IDs from shared manager
+        await self.upload_manager.upload_for_tool("file-search")
+        file_ids = self.upload_manager.get_files_for_tool("file-search")
+
+        if not file_ids:
+            logger.debug(
+                "No files for file-search, creating empty vector store"
+            )
+            return await self.create_vector_store_with_retry(
+                vector_store_name, max_retries, retry_delay
+            )
+
+        # Create vector store
+        vector_store_id = await self.create_vector_store_with_retry(
+            vector_store_name, max_retries, retry_delay
+        )
+
+        # Add files to vector store
+        await self._add_files_to_vector_store_with_retry(
+            vector_store_id, file_ids, max_retries, retry_delay
+        )
+
+        # Track uploaded files for cleanup
+        self.uploaded_file_ids.extend(file_ids)
+
+        logger.debug(
+            f"Created vector store {vector_store_id} with {len(file_ids)} files from shared manager"
+        )
+
+        return vector_store_id
+
     async def cleanup_resources(self) -> None:
         """Clean up uploaded files and created vector stores.
 
@@ -336,17 +402,54 @@ class FileSearchManager:
         self.created_vector_stores.clear()
 
     async def _cleanup_uploaded_files(self, file_ids: List[str]) -> None:
-        """Internal method to clean up specific file IDs.
+        """Clean up uploaded files with cache awareness."""
+        if not file_ids:
+            logger.debug("[fs] No uploaded files to clean up")
+            return
+
+        logger.debug(f"[fs] Cleaning up {len(file_ids)} uploaded files")
+
+        # Get cache and TTL configuration
+        cache = getattr(self.upload_manager, "_cache", None)
+        from .cache_config import DEFAULT_CACHE_TTL_DAYS
+
+        ttl_days = DEFAULT_CACHE_TTL_DAYS
+
+        if cache:
+            logger.debug(
+                f"[fs] Cache available, using TTL-aware cleanup (TTL: {ttl_days}d)"
+            )
+        else:
+            logger.debug("[fs] No cache available, using immediate cleanup")
 
-        Args:
-            file_ids: List of file IDs to delete
-        """
         for file_id in file_ids:
             try:
+                # Check if file should be preserved in cache
+                if cache and cache.is_file_cached_and_valid(file_id, ttl_days):
+                    logger.debug(f"[fs] Preserving cached file: {file_id}")
+                    # Update last accessed for LRU behavior
+                    cache.update_last_accessed(file_id)
+                    continue
+
+                # Delete the file
+                logger.debug(f"[fs] Deleting file: {file_id}")
                 await self.client.files.delete(file_id)
-                logger.debug(f"Cleaned up uploaded file: {file_id}")
+                logger.debug(f"[fs] Successfully deleted file: {file_id}")
+
             except Exception as e:
-                logger.warning(f"Failed to clean up file {file_id}: {e}")
+                if "404" in str(e) or "not found" in str(e).lower():
+                    logger.debug(
+                        f"[fs] File {file_id} already gone, cleaning cache"
+                    )
+                    # Clean up stale cache entry
+                    if cache:
+                        cache.invalidate_by_file_id(file_id)
+                else:
+                    logger.warning(
+                        f"[fs] Failed to delete file {file_id}: {e}"
+                    )
+
+        logger.debug("[fs] Cleanup complete")
 
     async def _cleanup_vector_stores(
         self, vector_store_ids: List[str]
